"""
	This class is designed to be an actor class i.e it will undertake certain actions given an event detected
	The platform has 2 main sections (detection & analysis).
	Action Types (Actors):
		- Alert : Sends an email or Webhook
		- Apps 	: Kill, Start
		- Folder: Archive, Delete (all, age, size)
        By design we are to understand that a message is structured as follows:
            {to,from,content} with content either being an arbitrary stream (or JSON)
	@TODO: 
		- upgrade to python 3.x
"""
import json
from threading import Thread
import os
import shutil
import subprocess
import re
from monitor import ProcessCounter
from utils.transport import QueueListener, QueueWriter, QueueReader
from utils.params import PARAMS
from ngram import NGram as ng
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

class Actor():
    @staticmethod
    def instance(name,args):
        """
            This function is a singleton that acts as a factory object for all the instances of this subclass
            @param name name of the class to instantiate
            @param args arguments to be passed in {configuration}
        """
        o = None
        try:
            o = eval("".join([name,"()"]))
            o.init(args)
        except Exception,e:
            print str(e)
        return o
    def __init__(self):        
        """
            Initializing the class with configuration. The configuration will be specific to each subclass
            @param args arguments the class needs to be configured
        """
        
        pass
    def getName(self):
        return self.__class__.__name__.lower()

    # def getIdentifier(self):
    #     return self.__class__.__name__.lower()

    def init(self,args):
        self.config = args
        
    def isValid(self,item):
        return False

    def execute(self,cmd):
        stream = None
        try:
            # subprocess.call (cmd,shell=False)
            out = subprocess.Popen(cmd,stdout=subprocess.PIPE)
            print out
            #stream = handler.communicate()[0]
        except Exception,e:
            pass      
    def post(self,**args):    
        pass
class Apps(Actor) :
    """
	    This class is designed to handle application, restart, if need be.
	    conf{app-name:{args}}
    """
    def __init__(self):
        Actor.__init__(self)
        self.ng = None
    def init(self,config) :
        """
            This function will initialize the the actor with applications and associated arguments
            @param args {"apps_o":"","app_x":params}
        """
        Actor.init(self,config)        
        self.ng = ng(self.config.keys())
    
    
    def can_start(self,name):
        """
            This function is intended to determine if it is possible to boot an application

        """
        items = self.ng.search(name) if self.ng is not None else []
        if len(items) == 0 :
            return False
        else:
            return items[0][1] > 0.01

    def startup(self,name) :
        """
            This function is intended to start a program given the configuration
        """        
        items   = self.ng.search(name)[0]
        app = items[0]
        args = self.config[app]
        
        cmd = " ".join([app,args,"&"    ])        
        self.execute([app,args])
        
    def kill(self,name) :
        """ 
            kill processes given the name, The function will not be case sensitive and partial names are accepted
            @NOTE: Make sure the reference to the app is not ambiguous
        """
        args = "".join(['ps -eo pid,command|grep -E -i "',name.lower(),'"|grep -E "^ {0,}[0-9]+" -o|xargs kill -9'])        
        #self.execute([args])
        subprocess.call([args],shell=True)
        
    def analyze(self,logs) :
        """
            This function is designed to analyze a few logs and take appropriate action
            @param logs logs of application/process data; folder analysis or sandbox analysis
        """
        for item in logs :
            name = item['label']
            if self.can_start(name) :
                self.startup(name)
            #
            

class Mailer (Actor):
    """
        This class is a mailer agent
    """
    def __init__(self):
        Actor.__init__(self)
    """
        conf = {uid:<account>,host:<host>,port:<port>,password:<password>}
    """
    def init(self,conf) :
        self.uid = conf['uid']


        try:

            self.handler = smtplib.SMTP_SSL(conf['host'],conf['port'])
            r = self.handler.login(self.uid,conf['password'])
            #
            # @TODO: Check the status of the authentication
            # If not authenticated the preconditions have failed
            #
        except Exception,e:
            print str(e)
            self.handler = None
            pass


    def send(self,**args) :
        subject = args['subject']
        message = args['message']
        to	= args['to']
        if '<' in message and '>' in message :
            message = MIMEText(message,'html')
        else:
            message = MIMEText(message,'plain')
        message['From'] = self.uid
        message['To']	= to
        message['Subject'] = subject
        return self.handler.sendmail(self.uid,to,message.as_string())
    def close(self):
        self.handler.quit()

            
class Folders(Actor):
    def __init__(self):
        Actor.__init__(self)
    """
        This is designed to handle folders i.e cleaning/archiving the folders
        if the user does NOT have any keys to cloud-view than she will not be able to archive
        {threshold:value}
        @params threshold   in terms of size, or age. It will be applied to all folders
    """
    
    def init(self,args):
        #self.lfolders   = args['folders'] #config['folders']
        #self.action     = args['action'] #{clear,archive} config['actions']['folders']
        self.threshold  = self.get_size( args['threshold']) #self.config['threshold'])
        
    
    def archive(self,item):
        """
            This function will archive all files in a given folder
            @pre : isValid
        """
        folder = item['label']
	name = folder.split(os.sep)
	name = name[len(name)-1]
        signature='-'.join([name,str(item['date']),str(item['count']),'files'])
        tarball=os.sep.join([folder,'..',signature])
        shutil.make_archive(tarball,'tar',folder)
        self.clean(item)
        #
        # @TODO: The archive can be uploaded to the cloud or else where
        #   @param id   cloud service idenfier {dropbox,box,google-drive,one-drive}
        #   @param key  authorization key for the given service
        #
        pass
    
    def clean(self,item):
        """
            This function consists in deleting files from a given folder
        """
        rpath = item['label']
        files = os.listdir(item['label'])
        for name in list(files) :
            path = os.sep.join([item['label'],name])
            if os.path.isdir(path) :
                shutil.rmtree(path)
            else:
                os.remove(path)
        #
        # 

    def get_size(self,value):
        """
            converts size values into MB and returns the value without units
        """
        units = {'MB':1000,'GB':1000000,'TB':1000000000} # converting to kb
        key = set(units.keys()) & set(re.split('(\d+)',value.replace(' ','').upper()))
        
        
        if len(key) == 0:
            return -1
        key = key.pop()
        return float(value.upper().replace('MB','').strip()) * units[key]
    
    def can_clean(self,item):        
        """
            This function returns whether the following :
            p : folder exists
            q : has_reached threashold
        """
        p = os.path.exists(item['label']) and item['label'] in self.lfolders    
        q = self.get_size(item['size']) >= self.threshold
        return p and q
    
    def analyze(self,logs):
        r = {'clean':self.clean,'archive':self.archive}
        self.lfolders = [ folder['label'] for folder in logs]
        for item in logs :
            if self.can_clean(item) :
                self.archive(item)
                #self.clean(item)
